library(shiny)
library(Seurat)
library(shinyFiles)
library(ggplot2)

# Define UI for application
ui <- fluidPage(

    # Application title
    titlePanel("scMonitor"),

    sidebarLayout(
        sidebarPanel(
          shinyFilesButton("file", label="Select File", title="Select a Seurat Object", multiple=F),
          uiOutput('groupby'),
          uiOutput('facetby'),
          textInput('features', label = "Genes"),
          actionButton("goButton", "Query"),
          sliderInput("height", "Altura", min=1000, max=20000, step=1000, value=6000)
        ),

        mainPanel(
           plotOutput("umapPlot"),
           uiOutput("featPlotUI")
        )
    )
)

# Define server logic required to draw a histogram
server <- function(input, output) {
  dades<-reactiveValues()
  dades$seu<-NULL
  
  volumes <- c(Home = fs::path_home(), "R Installation" = R.home(), getVolumes()())
  shinyFileChoose(input, 'file', roots=volumes)
  
  observe({
    if (!is.integer(input$file)){
      dades$seu<-readRDS(parseFilePaths(volumes, input$file)$datapath)
    }
  })
  
  output$groupby<-renderUI({
    if (!is.null(dades$seu)){
      selectInput("groupby", "GroupBy", c("Default", colnames(dades$seu@meta.data)))
    }
  })
  
  output$facetby<-renderUI({
    if (!is.null(dades$seu)){
      selectInput("facetby", "FacetBy", c("None", colnames(dades$seu@meta.data)))
    }
  })
  
  query<-eventReactive(input$goButton,{
    dades$fet_query<-strsplit(input$features, " ")[[1]]
  })
  
  output$umapPlot <- renderPlot({
    observeEvent(dades$seu, {})
    if (!is.null(dades$seu)){
 
    plot<-DimPlot(dades$seu, 
                  group.by = if(input$groupby != "Default"){input$groupby}else{NULL},
                  split.by=if(input$facetby != "None"){input$facetby}else{NULL})+
      theme(aspect.ratio=1)
    plot
    }
  })

  output$featPlotUI<- renderUI({
    observeEvent(dades$fet_query, {})
    if (!is.null(dades$seu)){
      plotOutput("featPlot", 
                 height = paste0(input$height/10,"px"))
    }
  })
  
  output$featPlot<-renderPlot({
    observeEvent(dades$seu, {})
    if (!is.null(dades$seu)){
      query()
      
      FeaturePlot(dades$seu,
                  split.by=if(input$facetby != "None"){input$facetby}else{NULL},
                  features = dades$fet_query)&theme(aspect.ratio = 1)
    }
  })
}

# Run the application 
shinyApp(ui = ui, server = server)
